#!/bin/sh
import os
import subprocess

modeset = ["sort", "total", "multi-total"]
dirset = ["heap", "qs"]
coreset = [4,8,12]
dataset = ["ijcnn1", "a9a", "real-sim", "rcv1_train.binary", "news20.binary"]

ROOT_PATH="./"
DATA_PATH="../liblinear-test/data/"
LOG_PATH=ROOT_PATH+"log/"
OUT_PATH=ROOT_PATH+"out/"
MOD_PATH=ROOT_PATH+"model/"
train="/train -s 21"

def parse(data, mode,core):
    if (mode != "multi-total"):
        logfile = LOG_PATH+data+"-"+mode+".log"
        out = OUT_PATH+data+"-"+mode+"-output.log"
    else:
        logfile = LOG_PATH+data+"-"+mode+str(core)+".log"
        out = OUT_PATH+data+"-"+mode+str(core)+"-output.log"

    fl = open(logfile, "r")
    fl2 = open(out, "w")

    timesum = {"qs":0., "heap":0.}
    timeavg = {"qs":0., "heap":0.}
    i = {"qs":0., "heap":0.}
    algo = "qs"
    for line in fl:
        sp = line.replace('\n', ' ')
        sp = sp.replace(',', ' ')
        sp = sp.split(' ')

        if mode == "sort":
            if sp[0] == "Heap":
                fl2.write("Heap: \n")
                algo = "heap"
            elif sp[0] == "Quickselect":
                fl2.write("Quickselect: \n")
            elif sp[0] == "sort_time":
                i[algo] += 1
                timesum[algo] += float(sp[2])
                fl2.write(str(i[algo]) +" "+ sp[2]+"\n")
        else:
            if sp[0] == "Heap":
                fl2.write("Heap: \n")
                algo = "heap"
            elif sp[0] == "Quickselect":
                fl2.write("Quickselect: \n")
            elif sp[0] == "train":
                i[algo] += 1
                timesum[algo] += float(sp[3])
                fl2.write(str(i[algo]) +" "+ sp[3]+"\n")

    timeavg["qs"] = timesum["qs"]/i["qs"] 
    timeavg["heap"] = timesum["heap"]/i["heap"] 
    reduction = (1-timeavg["qs"]/timeavg["heap"])*100          
    if mode != "multi-total":
        fl2.write("average "+mode+ " time using quickselect = "+ str(timeavg["qs"])+"\n")
        fl2.write("average "+mode+ " time using heap = "+ str(timeavg["heap"])+"\n")
        fl2.write(f"reduction percentage = {reduction:.2f}%")
    else:
        fl2.write("average "+mode+ " time using quickselect with "+ str(core) +" cores = "+ str(timeavg["qs"])+"\n")
        fl2.write("average "+mode+ " time using heap with "+ str(core) +" cores = "+ str(timeavg["heap"])+"\n")
        fl2.write(f"reduction percentage = {reduction:.2f}%") 
for data in dataset:
    for mode in modeset:
            if(mode == "sort"):
                filename = LOG_PATH+data+"-"+mode +".log"
                logfile = open(filename, "w") 
                comd = ROOT_PATH+"script/run_time.sh "+DATA_PATH+data+" "+MOD_PATH+data+".model"+" >> "+filename
                p = subprocess.Popen(['sh'] + comd.split(' '), stdout=subprocess.PIPE)

                logfile.write("Running "+data+ "\n")
                print("Sorting time of "+data)

                out, err = p.communicate()
                logfile.write(out.decode('ascii'))
                logfile.close()
                parse(data, mode, 0)
            elif(mode == "total"):
                filename = LOG_PATH+data+"-"+mode +".log"
                logfile = open(filename, "w")
                comd = ROOT_PATH+"script/mc_time.sh "+"1 "+DATA_PATH+data+" "+MOD_PATH+data+".model"+" >> "+filename
                p = subprocess.Popen(['sh']+comd.split(' '), stdout=subprocess.PIPE)

                logfile.write("Running "+data+ "\n")
                print("Total running time of "+data)

                out, err = p.communicate()
                logfile.write(out.decode('ascii'))
                logfile.close()
                parse(data, mode, 1)             

            elif(mode == "multi-total"):
                for core in coreset:    
                    filename = LOG_PATH+data+"-"+mode+str(core) +".log"
                    logfile = open(filename, "w")                          
                    comd = ROOT_PATH+"script/mc_time.sh "+str(core)+" "+DATA_PATH+data+" "+MOD_PATH+data+".model"+" >> "+filename
                    p = subprocess.Popen(['sh']+comd.split(' '), stdout=subprocess.PIPE)

                    logfile.write("Running "+data+" with "+str(core)+" cores\n")
                    print("Total running time of "+data + " with " +str(core)+" cores")

                    out, err = p.communicate()
                    logfile.write(out.decode('ascii'))
                    logfile.close()
                    parse(data, mode, core)




